# This sample was generated for the following code mutation detected by mutmut:
# 
# --- executing/_position_node_finder.py
# +++ executing/_position_node_finder.py
# @@ -95,7 +95,7 @@
#      "<<": ast.LShift,
#      ">>": ast.RShift,
#      "&": ast.BitAnd,
# -    "^": ast.BitXor,
# +    "XX^XX": ast.BitXor,
#      "|": ast.BitOr,
#  }
#  
# 

other ^ self