# This sample was generated for the following code mutation detected by mutmut:
# 
# --- executing/_position_node_finder.py
# +++ executing/_position_node_finder.py
# @@ -87,7 +87,7 @@
#      "**": ast.Pow,
#      "*": ast.Mult,
#      "@": ast.MatMult,
# -    "//": ast.FloorDiv,
# +    "XX//XX": ast.FloorDiv,
#      "/": ast.Div,
#      "%": ast.Mod,
#      "+": ast.Add,
# 

max_rows // 2