# This sample was generated for the following code mutation detected by mutmut:
# 
# --- executing/_position_node_finder.py
# +++ executing/_position_node_finder.py
# @@ -89,7 +89,7 @@
#      "@": ast.MatMult,
#      "//": ast.FloorDiv,
#      "/": ast.Div,
# -    "%": ast.Mod,
# +    "XX%XX": ast.Mod,
#      "+": ast.Add,
#      "-": ast.Sub,
#      "<<": ast.LShift,
# 

'<%s>' % i